"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.rmCommand = exports.addCommand = exports.addCommandLegacy = void 0;
const httpie_1 = require("httpie");
const colors_1 = require("kleur/colors");
const path_1 = __importDefault(require("path"));
const logger_1 = require("../logger");
const util_1 = require("../util");
const util_2 = require("../sources/util");
function pkgInfoFromString(str) {
    const idx = str.lastIndexOf('@');
    if (idx <= 0)
        return [str];
    return [str.slice(0, idx), str.slice(idx + 1)];
}
async function addCommandLegacy(addValue, commandOptions) {
    const { lockfile, config } = commandOptions;
    const packageOptions = config.packageOptions;
    let [pkgName, pkgSemver] = pkgInfoFromString(addValue);
    const installMessage = pkgSemver ? `${pkgName}@${pkgSemver}` : pkgName;
    logger_1.logger.info(`fetching ${colors_1.cyan(installMessage)} from CDN...`);
    if (!pkgSemver || pkgSemver === 'latest') {
        const { data } = await httpie_1.send('GET', `http://registry.npmjs.org/${pkgName}/latest`);
        pkgSemver = `^${data.version}`;
    }
    logger_1.logger.info(`Adding ${colors_1.cyan(colors_1.underline(`${pkgName}@${pkgSemver}`))} to your project lockfile. ${colors_1.dim(`(${util_1.LOCKFILE_NAME})`)}`);
    const addedDependency = { [pkgName]: pkgSemver };
    const lookupResponse = await util_1.remotePackageSDK.lookupBySpecifier(pkgName, pkgSemver);
    if (lookupResponse.error) {
        throw new Error(`There was a problem looking up ${pkgName}@${pkgSemver}`);
    }
    const newLockfile = util_1.convertSkypackImportMapToLockfile({
        ...lockfile === null || lockfile === void 0 ? void 0 : lockfile.dependencies,
        ...addedDependency,
    }, await util_1.remotePackageSDK.generateImportMap(addedDependency, lockfile ? util_1.convertLockfileToSkypackImportMap(packageOptions.origin, lockfile) : undefined));
    await util_1.writeLockfile(path_1.default.join(config.root, util_1.LOCKFILE_NAME), newLockfile);
    await util_2.getPackageSource(config).prepare();
}
exports.addCommandLegacy = addCommandLegacy;
async function addCommand(addValue, commandOptions) {
    const { lockfile, config } = commandOptions;
    if (config.packageOptions.source === 'remote') {
        return addCommandLegacy(addValue, commandOptions);
    }
    if (config.packageOptions.source === 'local') {
        throw new Error(`"snowpack add" only works when "packageOptions.source" is set.`);
    }
    let [pkgName, pkgSemver] = pkgInfoFromString(addValue);
    if (!pkgSemver || pkgSemver === 'latest') {
        const { data } = await httpie_1.send('GET', `http://registry.npmjs.org/${pkgName}/latest`);
        pkgSemver = `^${data.version}`;
    }
    const newLockfile = {
        dependencies: {
            ...(lockfile ? lockfile.dependencies : {}),
            [pkgName]: pkgSemver,
        },
        lock: lockfile ? lockfile.lock : {},
    };
    await util_1.writeLockfile(path_1.default.join(config.root, util_1.LOCKFILE_NAME), newLockfile);
    await util_2.getPackageSource(config).prepare();
    logger_1.logger.info(`Adding ${colors_1.cyan(colors_1.underline(`${pkgName}@${pkgSemver}`))} to your project lockfile. ${colors_1.dim(`(${util_1.LOCKFILE_NAME})`)}`);
}
exports.addCommand = addCommand;
async function rmCommandLegacy(rmValue, commandOptions) {
    var _a;
    const { lockfile, config } = commandOptions;
    const packageOptions = config.packageOptions;
    let [pkgName] = pkgInfoFromString(rmValue);
    logger_1.logger.info(`Removing ${colors_1.cyan(pkgName)} from project lockfile...`);
    const newLockfile = util_1.convertSkypackImportMapToLockfile((_a = lockfile === null || lockfile === void 0 ? void 0 : lockfile.dependencies) !== null && _a !== void 0 ? _a : {}, await util_1.remotePackageSDK.generateImportMap({ [pkgName]: null }, lockfile ? util_1.convertLockfileToSkypackImportMap(packageOptions.origin, lockfile) : undefined));
    delete newLockfile.dependencies[pkgName];
    await util_1.writeLockfile(path_1.default.join(config.root, util_1.LOCKFILE_NAME), newLockfile);
    await util_2.getPackageSource(config).prepare();
}
async function rmCommand(rmValue, commandOptions) {
    const { lockfile, config } = commandOptions;
    if (config.packageOptions.source === 'remote') {
        return rmCommandLegacy(rmValue, commandOptions);
    }
    if (config.packageOptions.source === 'local') {
        throw new Error(`"snowpack rm" only works when "packageOptions.source" is set.`);
    }
    let [pkgName] = pkgInfoFromString(rmValue);
    const newLockfile = {
        dependencies: { ...(lockfile ? lockfile.dependencies : {}) },
        lock: lockfile ? lockfile.lock : {},
    };
    delete newLockfile.dependencies[pkgName];
    await util_1.writeLockfile(path_1.default.join(config.root, util_1.LOCKFILE_NAME), newLockfile);
    await util_2.getPackageSource(config).prepare();
    logger_1.logger.info(`Removing ${colors_1.cyan(pkgName)} from project lockfile...`);
    const newLockfileAfterPrepare = await util_1.readLockfile(config.root);
    if (newLockfileAfterPrepare === null || newLockfileAfterPrepare === void 0 ? void 0 : newLockfileAfterPrepare.dependencies[pkgName]) {
        throw new Error(`Tried to remove dependency ${colors_1.cyan(pkgName)}, but its still used within your project.`);
    }
}
exports.rmCommand = rmCommand;
