"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPackageSource = exports.clearCache = void 0;
const local_1 = require("./local");
const remote_1 = require("./remote");
async function clearCache() {
    return Promise.all([local_1.PackageSourceLocal.clearCache(), remote_1.PackageSourceRemote.clearCache()]);
}
exports.clearCache = clearCache;
/**
 * Previously, all PackageSources were global. The new PackageSourceLocal is designed
 * to allow for uniqueness across projects / Snowpack instances. That support will come later,
 * so for now we just keep a global instance here.
 */
let sharedPackageSourceLocal;
let sharedPackageSourceRemote;
function getPackageSource(config) {
    if (config.packageOptions.source === 'remote') {
        sharedPackageSourceRemote = sharedPackageSourceRemote || new remote_1.PackageSourceRemote(config);
        return sharedPackageSourceRemote;
    }
    sharedPackageSourceLocal = sharedPackageSourceLocal || new local_1.PackageSourceLocal(config);
    return sharedPackageSourceLocal;
}
exports.getPackageSource = getPackageSource;
